#! /usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("util"));
const vm_1 = require("vm");
const child_process_1 = require("child_process");
const list_all_packages_1 = require("./utils/list-all-packages");
const find_monorepo_root_1 = require("./utils/find-monorepo-root");
const [expr, ...execCommandArgs] = process.argv.slice(2);
let useLernaExec = false;
if (execCommandArgs.includes('--lerna-exec')) {
    execCommandArgs.splice(execCommandArgs.indexOf('--lerna-exec'), 1);
    useLernaExec = true;
}
async function filterPackagesByExpression(expression) {
    const packages = [];
    for await (const { packageJson } of (0, list_all_packages_1.listAllPackages)()) {
        try {
            if ((0, vm_1.runInContext)(expression, (0, vm_1.createContext)(packageJson)))
                packages.push(packageJson.name);
        }
        catch {
        }
    }
    return packages;
}
async function lernaExec(packages) {
    const scope = packages.map((name) => `--scope=${name}`);
    if (scope.length === 0) {
        console.info(`No packages matched filter "${expr}"`);
        return;
    }
    const lernaBin = path_1.default.resolve(await (0, find_monorepo_root_1.findMonorepoRoot)(), 'node_modules', '.bin', 'lerna');
    (0, child_process_1.execFileSync)(lernaBin, ['exec', ...scope, ...execCommandArgs], {
        stdio: 'inherit',
    });
}
async function npmWorkspaces(packages) {
    const npmVersion = (0, child_process_1.execFileSync)('npm', ['-v']).toString();
    if (Number(npmVersion.substr(0, 2)) < 7) {
        throw Error(`"npm run where" relies on npm@7 features, using npm@${npmVersion}. Update npm to 7 or use the command with --lerna-exec instead`);
    }
    const workspaces = packages.map((name) => `--workspace=${name}`);
    if (workspaces.length === 0) {
        console.info(`No packages matched filter "${expr}"`);
        return;
    }
    console.log();
    console.log('Running "npm %s" for the following packages:', execCommandArgs.join(' '));
    console.log();
    console.log(util_1.default.inspect(packages));
    console.log();
    (0, child_process_1.execFileSync)('npm', [...workspaces, ...execCommandArgs], {
        stdio: 'inherit',
    });
}
async function main() {
    const packages = await filterPackagesByExpression(expr);
    if (useLernaExec) {
        await lernaExec(packages);
    }
    else {
        await npmWorkspaces(packages);
    }
}
process.on('unhandledRejection', (err) => {
    console.error();
    console.error(err?.stack || err?.message || err);
    process.exitCode = 1;
});
main().catch((err) => process.nextTick(() => {
    throw err;
}));
//# sourceMappingURL=where.js.map